#!/usr/bin/python2.7
# -*- coding: utf-8 -*-

from __future__ import division
try:
	import sys, os, getopt, math, qmig
except ImportError as inst:
	print "nomodule"
	exit()

__LANGS = ['ru_RU', 'en_EN']
__DEFAULT_LANG = 'en_EN'

__QM_DIR = os.path.dirname(os.path.abspath(__file__)) + '/'

__SOURCES_DIR = __QM_DIR+'sources/'
__DEFAULT_OUTPUT = __QM_DIR+'output/'

__OPTIONS = {
	'panacea': {
		'name': {
			'ru_RU': 'Панацея',
			'en_EN': 'Panacea'
		},
		'aliases': ['-p', '--pana', '--panacea'],
		'images': ['panacea_100.png'],
		'text': ['4,1,4,2,4,3,2,4,1,3,2,4,1,3,1,1,2,1,1,3,1,1,2,3,2,4,3,3,4,4,4,1,2,1,3,3,4,2,2,4', u'АИТ\nВЮО\nОНЯ']
	},
	'panacea_video': {
		'name': {
			'ru_RU': 'Панацея из видео',
			'en_EN': 'Panacea video'
		},
		'aliases': ['--pv', '--pvideo', '--panav', '--panavideo', '--panaceav', '--panaceavideo'],
		'images': ['panacea_100.png']
	},
	'panacea_matrix': {
		'name': {
			'ru_RU': 'Панацея матрица',
			'en_EN': 'Panacea matrix'
		},
		'aliases': ['--pm', '--pmatrix', '--panam', '--panamatrix', '--panaceam', '--panaceamatrix'],
		'text': [u'АИТ\nВЮО\nОНЯ']
	},
	'panacea_code': {
		'name': {
			'ru_RU': 'Панацея цифровой код',
			'en_EN': 'Panacea code'
		},
		'aliases': ['--pcode', '--panac', '--panacode', '--panaceac', '--panaceacode'],
		'text': ['4,1,4,2,4,3,2,4,1,3,2,4,1,3,1,1,2,1,1,3,1,1,2,3,2,4,3,3,4,4,4,1,2,1,3,3,4,2,2,4']
	},
	'panacea_youth': {
		'name': {
			'ru_RU': 'Панацея-молодость',
			'en_EN': 'Panacea-youth'
		},
		'aliases': ['--py', '--pyouth', '--pmolodost', '--panay', '--panayouth', '--panamolodost', '--panaceay', '--panaceayouth', '--panaceamolodost'],
		'images': ['molodost.png']
	},
	'bj1_lite': {
		'name': {
			'ru_RU': 'Браслет Жизни 1.0 Lite',
			'en_EN': 'Life Bracelet 1.0 Lite'
		},
		'aliases': ['--bj1', '--lb1', '--bj1l', '--bj1lite', '--lb1l', '--lb1lite'],
		'text': ['EPA\nRZA\nASL']
	},
	'bj21': {
		'name': {
			'ru_RU': 'Браслет Жизни 2.1',
			'en_EN': 'Life Bracelet 2.1'
		},
		'aliases': ['--bj21', '--lb21'],
		'images': ['2.1.png']
	},
	'bj30_lite': {
		'name': {
			'ru_RU': 'Браслет Жизни 3.0 Lite',
			'en_EN': 'Life Bracelet 3.0 Lite'
		},
		'aliases': ['--bj30l', '--bj30lite', '--lb30l', '--lb30lite', '--bj3l', '--bj3lite', '--lb3l', '--lb3lite'],
		'text': ['SXA\nOVR\nSLO']
	},
	'bj30_normal': {
		'name': {
			'ru_RU': 'Браслет Жизни 3.0 Normal',
			'en_EN': 'Life Bracelet 3.0 Normal'
		},
		'aliases': ['--bj30n', '--bj30normal', '--lb30n', '--lb30normal', '--bj3n', '--bj3normal', '--lb3n', '--lb3normal'],
		'images': ['3.0.2.png', '3.0.3.png', '3.0.1.png']
	},
	'oz': {
		'name': {
			'ru_RU': 'ОЗ',
			'en_EN': 'Special Zone'
		},
		'aliases': ['--oz', '--sz', '--specialzone'],
		'images': ['oz_g.png', 'oz1.2.jpg'],
		'text': ['KZO\nMAA\nAEO']
	},
	'oz_video': {
		'name': {
			'ru_RU': 'ОЗ из видео',
			'en_EN': 'Special Zone video'
		},
		'aliases': ['--ozv', '--ozvideo', '--szv', '--szvideo', '--specialzonev', '--specialzonevideo'],
		'images': ['oz_g.png']
	},
	'oz_matrix': {
		'name': {
			'ru_RU': 'ОЗ матрица',
			'en_EN': 'Special Zone matrix'
		},
		'aliases': ['--ozm', '--ozmatrix', '--szm', '--szmatrix', '--specialzonem', '--specialzonematrix'],
		'text': ['KZO\nMAA\nAEO']
	},
	'oz3': {
		'name': {
			'ru_RU': 'ОЗ 3',
			'en_EN': 'Special Zone 3'
		},
		'aliases': ['--oz3', '--sz3', '--specialzone3'],
		'images': ['oz1.2.jpg']
	},
	'spectr': {
		'name': {
			'ru_RU': 'Спектр',
			'en_EN': 'Spectrum'
		},
		'aliases': ['--spectr', '--spektr', '--spectrum'],
		'images': ['spectr_g.png'],
		'text': ['ZAS\nFRA\nKOA']
	},
	'spectr_video': {
		'name': {
			'ru_RU': 'Спектр из видео',
			'en_EN': 'Spectrum video'
		},
		'aliases': ['--spectrv', '--spektrv', '--spectrumv'],
		'images': ['spectr_g.png']
	},
	'spectr_matrix': {
		'name': {
			'ru_RU': 'Спектр матрица',
			'en_EN': 'Spectrum matrix'
		},
		'aliases': ['--spectrm', '--spektrm', '--spectrumm'],
		'text': ['ZAS\nFRA\nKOA']
	},
	'at': {
		'name': {
			'ru_RU': 'AT',
			'en_EN': 'AT'
		},
		'aliases': ['--at'],
		'images': ['at_in_g.png', 'at_out_g.png']
	},
	'at5': {
		'name': {
			'ru_RU': 'AT5',
			'en_EN': 'AT5'
		},
		'aliases': ['--at5'],
		'images': ['at5_graf.jpg']
	},
	'soul': {
		'name': {
			'ru_RU': 'Душа',
			'en_EN': 'Soul'
		},
		'aliases': ['--dusha', '--soul'],
		'images': ['dusha1_2_0.gif', 'dusha1_2_1.gif', 'dusha1_2_2.gif', 'dusha1_2_3.gif', 'dusha1_2_4.gif', 'dusha1_2_5.gif', 'dusha1_2_6.gif', 'dusha1_2_7.gif', 'dusha1_2_8.gif', 'dusha1_2_9.gif', 'dusha1_2_10.gif', 'dusha1_2_11.gif', 'dusha1_2_12.gif', 'dusha1_2_13.gif', 'dusha1_2_14.gif', 'dusha1_2_15.gif', 'dusha2_2_0.gif', 'dusha2_2_1.gif', 'dusha2_2_2.gif', 'dusha2_2_3.gif', 'dusha2_2_4.gif', 'dusha2_2_5.gif', 'dusha2_2_6.gif', 'dusha2_2_7.gif', 'dusha2_2_8.gif', 'dusha2_2_9.gif', 'dusha2_2_10.gif', 'dusha2_2_11.gif', 'dusha2_2_12.gif', 'dusha2_2_13.gif', 'dusha2_2_14.gif', 'dusha2_2_15.gif']
	},
	'innerchild': {
		'name': {
			'ru_RU': 'Внутренний ребёнок',
			'en_EN': 'Inner child'
		},
		'aliases': ['--innerchild', '--child'],
		'images': ['innerchild1_2_0.gif', 'innerchild1_2_1.gif', 'innerchild1_2_2.gif', 'innerchild1_2_3.gif', 'innerchild1_2_4.gif', 'innerchild1_2_5.gif', 'innerchild1_2_6.gif', 'innerchild1_2_7.gif', 'innerchild1_2_8.gif', 'innerchild1_2_9.gif', 'innerchild1_2_10.gif', 'innerchild1_2_11.gif', 'innerchild1_2_12.gif', 'innerchild1_2_13.gif', 'innerchild1_2_14.gif', 'innerchild1_2_15.gif', 'innerchild2_2_0.gif', 'innerchild2_2_1.gif', 'innerchild2_2_2.gif', 'innerchild2_2_3.gif', 'innerchild2_2_4.gif', 'innerchild2_2_5.gif', 'innerchild2_2_6.gif', 'innerchild2_2_7.gif', 'innerchild2_2_8.gif', 'innerchild2_2_9.gif', 'innerchild2_2_10.gif', 'innerchild2_2_11.gif', 'innerchild2_2_12.gif', 'innerchild2_2_13.gif', 'innerchild2_2_14.gif', 'innerchild2_2_15.gif']
	},
	'elf': {
		'name': {
			'ru_RU': 'Эльфийская',
			'en_EN': 'Elven'
		},
		'aliases': ['--elf', '--elven'],
		'images': ['elf_micro_1_0.gif', 'elf_micro_1_1.gif', 'elf_micro_1_2.gif', 'elf_micro_1_3.gif', 'elf_micro_1_4.gif', 'elf_micro_1_5.gif', 'elf_micro_1_6.gif', 'elf_micro_1_7.gif', 'elf_micro_2_0.gif', 'elf_micro_2_1.gif', 'elf_micro_2_2.gif', 'elf_micro_2_3.gif', 'elf_micro_2_4.gif', 'elf_micro_2_5.gif', 'elf_micro_2_6.gif', 'elf_micro_2_7.gif']
	},
	'etalon': {
		'name': {
			'ru_RU': 'Эталон',
			'en_EN': 'Etalon'
		},
		'aliases': ['--etalon'],
		'images': ['etalon_g.png']
	},
	'tablet': {
		'name': {
			'ru_RU': 'Таблетка',
			'en_EN': 'Tablet'
		},
		'aliases': ['--tablet', '--tabletka', '--pill'],
		'images': ['tablet_g.png']
	},
	'blood_cleaning': {
		'name': {
			'ru_RU': 'Чистка крови',
			'en_EN': 'Blood cleaning'
		},
		'aliases': ['--blood', '--bloodcleaning', '--krov', '--chistkakrovi'],
		'images': ['bloodclean_g.png']
	},
	'probiotic': {
		'name': {
			'ru_RU': 'Пробиотик',
			'en_EN': 'Probiotic'
		},
		'aliases': ['--probiotic', '--probiotik'],
		'images': ['probiotic_g.png']
	},
	'ozone': {
		'name': {
			'ru_RU': 'Озон',
			'en_EN': 'Ozone'
		},
		'aliases': ['--ozon', '--ozone'],
		'images': ['ozone_g.png']
	},
	'wheel': {
		'name': {
			'ru_RU': 'Колесо',
			'en_EN': 'Wheel'
		},
		'aliases': ['--wheel', '--koleso'],
		'images': ['wheel_g.png']
	},
	'whirligig': {
		'name': {
			'ru_RU': 'Волчок',
			'en_EN': 'Whirligig'
		},
		'aliases': ['--whirligig', '--volchok'],
		'images': ['whirligig_g.jpg']
	},
	'stopmagic': {
		'name': {
			'ru_RU': 'Стоп-магия',
			'en_EN': 'Stop magic'
		},
		'aliases': ['--sm', '--stopmagic', '--stop', '--stopm', '--stopmagiya'],
		'images': ['sm1_g.jpg', 'sm2_g.jpg']
	},
	'health': {
		'name': {
			'ru_RU': 'Здоровье',
			'en_EN': 'Health'
		},
		'aliases': ['--health', '--zdorovie'],
		'images': ['health_g.jpg']
	},
	'protection': {
		'name': {
			'ru_RU': 'Защита',
			'en_EN': 'Protection'
		},
		'aliases': ['--protection', '--zaschita', '--zashita', '--zachita'],
		'images': ['protection_g.jpg']
	},
	'stabilization': {
		'name': {
			'ru_RU': 'Стабилизация',
			'en_EN': 'Stabilization'
		},
		'aliases': ['--stab', '--stabilization', '--stabil', '--stabilizatsiya'],
		'images': ['adt_stab_g.jpg']
	},
	'pc': {
		'name': {
			'ru_RU': 'ПК',
			'en_EN': 'PC'
		},
		'aliases': ['--pc', '--pk'],
		'images': ['micro_pk.gif']
	},
	'amb': {
		'name': {
			'ru_RU': 'АМБ',
			'en_EN': 'AMB'
		},
		'aliases': ['--amb'],
		'text': ['''7,3,7,3,9,0,0,0,0,0,7,0,7,2,6,3,0,2,3,7,3,2,3,5,0,5,2,5,8,5,0,2,6,5,5,7,0,2,3,5,5,2,2,7,6,5,9,4,3,9,4,6,3,1,9,2,3,4,5,0,0,8,4,9,
4,9,0,9,4,1,0,2,9,5,4,5,0,9,3,3,5,3,6,2,0,7,9,7,3,2,9,9,3,1,3,2,6,0,3,0,1,0,2,7,9,3,8,4,7,7,0,5,9,5,8,4,0,8,4,6,1,2,9,6,1,4,8,5,
6,6,2,6,7,3,3,2,0,2,8,0,4,3,8,4,5,4,9,1,1,5,1,7,9,5,0,3,5,2,5,7,4,0,7,1,2,4,4,2,4,3,7,7,6,0,0,3,4,1,0,8,5,4,7,7,1,2,3,9,5,8,3,3,
9,1,8,0,2,9,0,1,2,7,1,2,8,0,2,0,3,3,5,2,7,8,6,2,7,3,8,5,2,3,9,0,2,4,0,0,8,4,2,0,8,9,1,8,6,9,1,3,5,0,5,1,6,7,8,4,4,3,5,7,2,6,7,0,
1,9,8,9,9,1,0,3,1,8,0,0,7,9,0,7,4,4,3,5,7,1,6,6,9,4,3,8,5,8,8,0,8,8,3,4,4,8,4,0,1,0,5,6,1,9,4,8,9,0,7,7,5,3,7,1,7,9,0,5,5,1,2,3,
3,3,5,4,9,5,5,5,6,2,1,1,5,5,9,9,4,7,3,9,9,3,7,4,9,3,9,4,5,1,5,3,8,7,1,5,2,1,0,3,5,4,0,1,3,1,1,3,5,4,0,7,2,6,8,7,6,1,6,3,9,5,1,6,
8,5,5,1,0,4,0,2,0,8,1,6,2,8,0,7,7,9,1,1,5,0,4,4,1,6,8,1,0,9,6,4,6,1,6,3,7,9,5,6,8,4,6,6,5,1,8,4,6,6,9,6,0,6,6,6,3,3,4,7,7,7,1,0,
6,8,0,0,8,3,1,1,2,2,2,8,1,5,1,5,5,8,0,6,3,7,0,9,1,1,0,4,9,4,4,8,5,9,9,2,3,2,9,5,9,7,3,9,9,8,6,0,3,1,0,9,6,4,0,2,6,6,4,4,6,3,9,5,
4,3,7,9,0,8,7,5,1,1,5,6,5,6,3,4,1,4,0,1,2,0,0,6,4,0,7,9,3,8,2,0,1,0,6,6,1,6,3,8,8,9,8,6,5,0,2,2,5,7,2,5,5,7,3,8,4,8,7,4,4,8,3,4,
1,6,4,1,6,9,4,0,2,4,5,4,2,0,3,5,8,1,1,9,1,8,1,7,9,2,4,9,2,1,6,8,0,5,9,4,5,8,5,1,3,3,0,7,6,2,9,9,7,1,5,7,6,2,6,0,5,2,7,6,8,4,6,6,
9,6,2,1,2,4,1,2,2,6,8,3,7,1,2,5,7,4,6,9,2,5,5,8,8,7,8,8,5,6,7,4,7,1,5,8,8,2,2,0,4,8,0,8,2,3,0,8,0,6,7,7,7,2,6,2,3,2,4,4,8,4,9,0,
8,0,3,4,4,1,4,5,6,4,1,2,2,1,9,5,6,7,9,9,8,6,3,3,9,8,7,2,4,3,0,7,3,1,8,1,1,9,1,5,8,7,5,4,1,6,3,3,3,1,1,4,7,3,9,5,8,7,5,6,0,1,2,4,
7,1,0,0,8,9,4,6,3,7,7,0,0,5,4,8,0,3,4,8,2,5,4,0,3,3,8,4,9,0,3,9,9,9,8,1,5,9,0,1,0,2,2,3,4,2,0,5,0,7,7,7,3,2,1,2,4,2,4,0,1,6,6,3,
3,8,7,0,0,8,9,8,7,3,7,8,0,0,5,0,9,2,2,6,1,2,0,3,5,5,7,3,6,5,2,1,9,3,8,5,7,7,2,9,3,3,7,0,6,6,5,3,7,0,1,8,3,7,8,8,1,0,6,5,8,6,8,5,
4,2,7,4,3,6,7,1,0,3,9,2,4,7,3,2,9,2,9,1,4,2,6,2,9,2,9,1,8,8,9,4,2,0,1,2,1,7,1,7,8,8,8,6,8,4,7,3,3,2,7,5,4,8,0,4,8,5,4,7,3,1,3,3,
7,4,7,8,1,1,7,9,4,1,9,9,1,0,6,6,4,7,0,1,4,9,9,5,6,7,8,8,9,9,8,9,8,2,7,2,9,4,3,9,2,7,2,8,7,4,9,1,0,3,1,8,1,0,9,6,6,0,0,6,7,3,8,3''']
	},
	'spec': {
		'name': {
			'ru_RU': 'Спецсредство',
			'en_EN': 'Specific '
		},
		'aliases': ['--spec', '--specific', '--special'],
		'images': ['/special_cleaner/spec00000.gif','/special_cleaner/spec00001.gif','/special_cleaner/spec00002.gif','/special_cleaner/spec00003.gif','/special_cleaner/spec00004.gif','/special_cleaner/spec00005.gif','/special_cleaner/spec00006.gif','/special_cleaner/spec00007.gif','/special_cleaner/spec00008.gif','/special_cleaner/spec00009.gif',
'/special_cleaner/spec00010.gif','/special_cleaner/spec00011.gif','/special_cleaner/spec00012.gif','/special_cleaner/spec00013.gif','/special_cleaner/spec00014.gif','/special_cleaner/spec00015.gif','/special_cleaner/spec00016.gif','/special_cleaner/spec00017.gif','/special_cleaner/spec00018.gif','/special_cleaner/spec00019.gif','/special_cleaner/spec00020.gif','/special_cleaner/spec00021.gif','/special_cleaner/spec00022.gif','/special_cleaner/spec00023.gif','/special_cleaner/spec00024.gif','/special_cleaner/spec00025.gif','/special_cleaner/spec00026.gif','/special_cleaner/spec00027.gif','/special_cleaner/spec00028.gif','/special_cleaner/spec00029.gif','/special_cleaner/spec00030.gif','/special_cleaner/spec00031.gif','/special_cleaner/spec00032.gif','/special_cleaner/spec00033.gif','/special_cleaner/spec00034.gif','/special_cleaner/spec00035.gif','/special_cleaner/spec00036.gif','/special_cleaner/spec00037.gif','/special_cleaner/spec00038.gif','/special_cleaner/spec00039.gif','/special_cleaner/spec00040.gif','/special_cleaner/spec00041.gif','/special_cleaner/spec00042.gif','/special_cleaner/spec00043.gif','/special_cleaner/spec00044.gif','/special_cleaner/spec00045.gif','/special_cleaner/spec00046.gif','/special_cleaner/spec00047.gif','/special_cleaner/spec00048.gif','/special_cleaner/spec00049.gif','/special_cleaner/spec00050.gif','/special_cleaner/spec00051.gif','/special_cleaner/spec00052.gif','/special_cleaner/spec00053.gif','/special_cleaner/spec00054.gif','/special_cleaner/spec00055.gif','/special_cleaner/spec00056.gif','/special_cleaner/spec00057.gif','/special_cleaner/spec00058.gif','/special_cleaner/spec00059.gif','/special_cleaner/spec00060.gif','/special_cleaner/spec00061.gif','/special_cleaner/spec00062.gif']
	}
}

__GETOPT_OPTIONS = []
for i in __OPTIONS:
	for j in __OPTIONS[i]['aliases']:
		if not j == '-p':
			__GETOPT_OPTIONS.append(j[2:])
		else:
			__GETOPT_OPTIONS.append(j[1:])

__DEFAULT_OPACITY = 10
__MIN_OPACITY = 1

try:
	gOpts, gArgs = getopt.getopt(sys.argv[1:], "f:o:p", ["file=","opacity=","lang=","output="]+__GETOPT_OPTIONS)
except getopt.GetoptError:
	print 'usage'
	exit()

opts = {}

for opt, arg in gOpts:
	opts[opt] = arg

filename = opts.get('-f') if opts.get('-f') else opts.get('--file')
if not os.path.isfile(filename):
	print 'filenotfound'
	exit()

output = opts.get('--output')
if not output:
	output = __DEFAULT_OUTPUT
if not output[-1] == '/':
	output += '/'
if not os.path.isdir(output):
	try:
		os.makedirs(output)
	except Exception:
		print 'dirnotfound'
		exit()
	if not os.path.isdir(output):
		print 'dirnotfound'
		exit()

opacity = opts.get('-o') if opts.get('-o') else opts.get('--opacity')
if opacity is None:
	opacity = __DEFAULT_OPACITY
else:
	try:
		opacity = int(opacity)
	except ValueError:
		opacity = __DEFAULT_OPACITY
	if opacity < 0 or opacity > 100:
		opacity = __DEFAULT_OPACITY
opacity = opacity / 100

lang = opts.get('--lang')
if lang is None or not lang in __LANGS:
	lang = __DEFAULT_LANG

try:
	for opt, arg in gOpts:
		for optIndex in __OPTIONS:
			if opt in __OPTIONS[optIndex]['aliases']:
				optionImages = __OPTIONS[optIndex].get('images')
				optionText = __OPTIONS[optIndex].get('text')
				imaged = False
				if optionImages:
					i = 0
					for optionImage in optionImages:
						if not i:
							processedImage = qmig.openImageRGBA(filename)
						else:
							processedImage = qmig.openImageRGBA(output+newfilename)
						overlayImage = qmig.createOverlayImageUseFilling(__SOURCES_DIR+optionImage, processedImage.size)
						resultImage = qmig.blendImages(processedImage, overlayImage, opacity)
						splitfilename = os.path.splitext(os.path.basename(filename))
						newfilename = splitfilename[0]+'+'+__OPTIONS[optIndex]['name'][lang]+splitfilename[1]
						try:
							resultImage.save(output+newfilename)
						except IOError:
							print 'cannotwritetodir'
							exit()
						if opacity / 1.5 >= __MIN_OPACITY / 100:
							opacity /= 1.5
						i += 1
					imaged = True
				if optionText:
					t = 0
					for optText in optionText:
						processedImage = qmig.openImageRGBA(filename if not imaged else output+newfilename)
						if optIndex == 'panacea_code' or (optIndex == 'panacea' and not t):
							fontsize = int(math.floor(processedImage.size[0]/40))+1
							for i in range(0, int(math.floor(processedImage.size[1]/(fontsize+5)))):
								qmig.printTextOnImage(processedImage, optText, (0, (fontsize+5)*i), opacity, fontsize)
						elif optIndex == 'amb':
							fontsize = int(math.floor(processedImage.size[0]/64))
							if processedImage.size[0] < fontsize*20:
								fontsize = int(math.floor(processedImage.size[0]/20))
							for i in range(0, int(math.floor(processedImage.size[0]/(fontsize*20+5)))):
								qmig.printTextOnImage(processedImage, optText, (0, (fontsize*20+5)*i), opacity, fontsize)
						else:
							fontsize = 10 if (processedImage.size[0] > 35 and processedImage.size[1] > 35) else 1
							for i in range(0, int(math.floor(processedImage.size[1] / (fontsize*3 + 15)))):
								for j in range(0, int(math.ceil(processedImage.size[0] / (fontsize*3 + 5)))):
									qmig.printTextOnImage(processedImage, optText, ((fontsize*3+5)*(j-1), ((fontsize*3+15)*i)), opacity, fontsize)
						splitfilename = os.path.splitext(os.path.basename(filename))
						newfilename = splitfilename[0]+'+'+__OPTIONS[optIndex]['name'][lang]+splitfilename[1]
						try:
							processedImage.save(output+newfilename)
						except IOError:
							print 'cannotwritetodir'
							exit()
						t += 1
				print 'done;'+newfilename
				exit()
except qmig.QMIGException as inst:
	print inst
	exit()