#!/usr/bin/python2.7
# -*- coding: utf-8 -*-

try:
	import sys, locale, getopt, os, subprocess, re, mimetypes, time, requests, shutil
except ImportError as inst:
	print inst
	exit()

__VERSION = '1.05'

__LANGS = ['ru_RU', 'en_EN']
__DEFAULT_LANG = 'en_EN'

__MESSAGES = {
	'start': {
		'ru_RU': 'Начинаю работу...',
		'en_EN': 'Starting...'
	},
	'usage': {
		'ru_RU': 'Некорректное использование. Запустите `./generator.py --help` для получения более подробной справки.',
		'en_EN': 'Incorrect usage. Run `./generator.py --help` for more information.'
	},
	'nomodule': {
		'ru_RU': 'Ошибка: не найден необходимый для работы модуль python. ',
		'en_EN': 'Error: required module not found. '
	},
	'filenotfound': {
		'ru_RU': 'Ошибка: файл не найден: ',
		'en_EN': 'Error: file not found: '
	},
	'cannotwritetofile': {
		'ru_RU': 'Ошибка: невозможно записать данные в файл: ',
		'en_EN': 'Error: can not write to file: '
	},
	'dirnotfound': {
		'ru_RU': 'Ошибка: директория не найдена: ',
		'en_EN': 'Error: directory not found and can not be created: '
	},
	'dirnotfoundandcannotbecreated': {
		'ru_RU': 'Ошибка: директория не найдена и не может быть создана: ',
		'en_EN': 'Error: directory not found and can not be created: '
	},
	'dirisempty': {
		'ru_RU': 'В директории нет ни одного файла: ',
		'en_EN': 'There are no files in the directory: '
	},
	'urlnotfound': {
		'ru_RU': 'Ошибка: URL не найден или не может быть открыт: ',
		'en_EN': 'Error: URL not found or can not be opened: '
	},
	'urlloading': {
		'ru_RU': 'Загружается файл: ',
		'en_EN': 'File loading: '
	},
	'cannotopenfile': {
		'ru_RU': 'Ошибка: не удалось открыть файл: ',
		'en_EN': 'Error: can not open file: '
	},
	'cannotwritetodir': {
		'ru_RU': 'Ошибка: не удалось записать данные в директорию: ',
		'en_EN': 'Error: can not write data to the directory: '
	},
	'inputcannotbeoutput': {
		'ru_RU': 'Ошибка: директория с файлам для обработки и директория для результатов не могут совпадать: ',
		'en_EN': 'Error: input and output directory can not be the same: '
	},
	'unsupportedformat': {
		'ru_RU': 'Ошибка: неподдерживаемый тип файла (поддерживаются только .gif, .jpg, .jpeg, .png): ',
		'en_EN': 'Error: unsupported file type (only .gif, .jpg, .jpeg, .png): '
	},
	'listfilenottxt': {
		'ru_RU': 'Ошибка: файл имеет не текстовый формат: ',
		'en_EN': 'Error: file is not in a text format: '
	},
	'opacity': {
		'ru_RU': 'Ошибка: некорректное значение прозрачности: ',
		'en_EN': 'Error: incorrect opacity value: '
	},
	'threads': {
		'ru_RU': 'Ошибка: некорректное значение количества потоков: ',
		'en_EN': 'Error: incorrect threads count value: '
	},
	'notfilenoturlnotcoords': {
		'ru_RU': 'Ошибка: значение не соответствует ни файлу, ни URL, ни координатам: ',
		'en_EN': 'Error: value does not match any file or the URL or coordinates: '
	},
	'incorrectsources': {
		'ru_RU': 'Ошибка: некорректно указаны источники спутниковых снимков: ',
		'en_EN': 'Error: incorrect sources of the sattelites: '
	},
	'incorrectzoom': {
		'ru_RU': 'Ошибка: некорректно указаны уровни масштабирования: ',
		'en_EN': 'Error: incorrect zoom levels: '
	},
	'nomaps': {
		'ru_RU': 'При использовании в качестве источника только Яндекс Карт при указанных уровнях масштабирования не будет загружено ни одного снимка: ',
		'en_EN': 'When using as a source Yandex Maps only with indicated zoom levels no photo will be uploaded: '
	},
	'sourcenotfound': {
		'ru_RU': 'Ошибка: не найден необходимый для обработки файл-исходник. Проверьте наличие папки sources и, при необходимости, загрузите исходники скриптом loadSources.sh',
		'en_EN': 'Error: source file not found. Check sources directory and if necessary download sources using loadSources.sh'
	},
	'default': {
		'ru_RU': 'Будет использовано значение по-умолчанию: ',
		'en_EN': 'Value is used by default: '
	},
	'unexpected': {
		'ru_RU': 'Непредвиденная ошибка: не удалось обработать файл: ',
		'en_EN': 'Unexpected error: unable to process file: '
	},
	'done': {
		'ru_RU': 'Файл обработан: ',
		'en_EN': 'Processed: '
	},
	'exit': {
		'ru_RU': 'Обработка завершена.',
		'en_EN': 'Done.'
	}
}

__VERSIONS = {
	'ru_RU': 'Генератор изображений QuantumMagic версии ',
	'en_EN': 'QuantumMagic Image Generator version '
}

__OPTIONS = ['--blood','--bloodcleaning','--krov','--chistkakrovi','--wheel','--koleso','--stab','--stabilization','--stabil','--stabilizatsiya','--bj30n','--bj30normal','--lb30n','--lb30normal','--bj3n','--bj3normal','--lb3n','--lb3normal','--amb','--pv','--pvideo','--panav','--panavideo','--panaceav','--panaceavideo','--sm','--stopmagic','--stop','--stopm','--stopmagiya','--protection','--zaschita','--zashita','--zachita','--bj1','--lb1','--bj1l','--bj1lite','--lb1l','--lb1lite','--at','--pcode','--panac','--panacode','--panaceac','--panaceacode','--whirligig','--volchok','--probiotic','--probiotik','--tablet','--tabletka','--pill','--innerchild','--child','--spectrm','--spektrm','--spectrumm','--pm','--pmatrix','--panam','--panamatrix','--panaceam','--panaceamatrix','--elf','--elven','--dusha','--soul','--ozon','--ozone','--pc','--pk','--at5','-p','--pana','--panacea','--bj30l','--bj30lite','--lb30l','--lb30lite','--bj3l','--bj3lite','--lb3l','--lb3lite','--bj21','--lb21','--etalon','--health','--zdorovie','--py','--pyouth','--pmolodost','--panay','--panayouth','--panamolodost','--panaceay','--panaceayouth','--panaceamolodost','--oz3','--sz3','--specialzone3','--spec','--specific','--special']
__GETOPT_OPTIONS = []
for i in __OPTIONS:
	__GETOPT_OPTIONS.append(i[2:])

__HELP = {
	'ru_RU': '''Использование: ./generator.py [список ключей]
Обрабатываемый объект (должен быть указан обязательно): файл, директория, URL, координаты, список (файлов, URL или координат).
	Файл (обрабатывается указанный файл): -f filename или --file=filename
	Директория (обрабатываются все файлы в указанной директории): -d dirname или --dir=dirname
	URL (обрабатывается файл по URL-адресу): -u URL или --url=URL
	Координаты (скачиваются и обрабатываются спутниковые снимки по указанным в формате ±12.345,±67.890 координатам): -c x,y или --coords=x,y
		дополнительно можно указать уровни масштабирования (по-умолчанию от 21 до 0): --zoom=from,to
		дополнительно можно указать источники снимков (по-умолчанию Google Maps и Яндекс Карты): --sources=yandex,google
	Список: -l filename или --list=filename;
		в файле-списке могут быть записаны имена файлов для обработки (каждое имя файла с новой строки),
		URL-адреса (каждый URL с новой строки), 
		координаты в формате ±12.345,±67.890 (каждая с новой строки).
Прозрачность (число от 0 до 100, по-умолчанию 10): -o [0-100] или --opacity=[0-100]
Количество потоков (число от 1 до 1024, по-умолчанию 10): -t [1-1024] или --threads=[1-1024]
Опции:
	Панацея полная (видео+матрица+код): -p или --pana или --panacea
		Панацея из видео: --pv или --pvideo или --panav или --panavideo или --panaceav или --panaceavideo
		Панацея матрица: --pm или --pmatrix или --panam или --panamatrix или --panaceam или --panaceamatrix
		Панацея цифровой код: --pcode или --panac или --panacode или --panaceac или --panaceacode
		Панацея-молодость: --py или --pyouth или --pmolodost или --panay или --panayouth или --panamolodost или --panaceay или --panaceayouth или --panaceamolodost
	Браслеты Жизни:
		Браслет Жизни 1.0 Lite: --bj1 или --lb1 или --bj1l или --bj1lite или --lb1l или --lb1lite
		Браслет Жизни 2.1: --bj21 или --lb21
	Браслеты Жизни 3.0:
		Браслет Жизни 3.0 Lite: --bj30l или --bj30lite или --lb30l или --lb30lite или --bj3l или --bj3lite или --lb3l или --lb3lite
		Браслет Жизни 3.0 Normal: --bj30n или --bj30normal или --lb30n или --lb30normal или --bj3n или --bj3normal или --lb3n или --lb3normal
		ОЗ полная (матрица+видео+ОЗ 3): --oz или --sz или --specialzone
			ОЗ матрица: --ozm или --ozmatrix или --szm или --szmatrix или --specialzonem или --specialzonematrix
			ОЗ из видео: --ozv или --ozvideo или --szv или --szvideo или --specialzonev или --specialzonevideo
			ОЗ 3: --oz3 или --sz3 или --specialzone3
	Работа с социумом:
		Спектр полный (матрица+видео): --spectr или --spektr или --spectrum
			Спектр матрица: --spectrm или --spektrm или --spectrumm
			Спектр из видео: --spectrv или --spektrv или --spectrumv
		AT: --at
		AT5: --at5
	Трансляции:
		Душа: --dusha или --soul
		Внутренний ребёнок: --innerchild или --child
		Эльфийская: --elf или --elven
	Спецсредство: --spec или --specific или --special
	Модули Браслетов Жизни:
		Эталон: --etalon
		Таблетка: --tablet или --tabletka или --pill
		Чистка крови: --blood или --bloodcleaning или --krov или --chistkakrovi
		Пробиотик: --probiotic или --probiotik
		Озон: --ozon или --ozone
	Управление энергией:
		Колесо: --wheel или --koleso
		Волчок: --whirligig или --volchok
	Программы стабилизатора:
		Стоп-магия: --sm или --stopmagic или --stop или --stopm или --stopmagiya
		Здоровье: --health или --zdorovie
		Защита: --protection или --zaschita или --zashita или --zachita
		Стабилизация: --stab или --stabilization или --stabil или --stabilizatsiya
	ПК:
		ПК: --pc или --pk
		АМБ: --amb
Директория для результатов обработки (по-умолчанию `output` внутри директории, где расположен генератор): --output=dirname
Язык (значение локали, по-умолчанию системная локаль): --lang=[locale]
Тихий режим (без вывода лога обработки): -s или --silent
Режим отладки (только для разработчиков): --debug
Вызов этой справки: -h или --help
Версия программы: -v или --version
Связь с разработчиком: saband@quantummagic.org
		''',
	'en_EN': '''Usage: ./generator.py [keys]
Processed object (required): file, directory, URL, coordinates, list (of files, URLs or coordinates).
	File (handled the specified file): -f filename or --file=filename
	Directory (handled all files in the specified directory): -d dirname or --dir=dirname
	URL (handled file by the URL): -u URL or --url=URL
	Coordinates (loaded and handled satellites by specified coordinates in the format ±12.345,±67.890): -c x,y or --coords=x,y
		additionally you can specify the zoom levels (by default from 21 to 0): --zoom=from,to
		additionally you can specify the satellites sources (by default Google Maps and Yandex Maps): --sources=yandex,google
	List: -l filename or --list=filename;
		in the list file can be specified names of files for processing (each file name on a new line),
		URLs (each URL on a new line), 
		coordinates in the format ±12.345,±67.890 (each recored on a new line).
Opacity (a number from 0 to 100, default 10): -o [0-100] or --opacity=[0-100]
Threads count (a number from 1 to 1024, default 10): -t [1-1024] or --threads=[1-1024]
Options:
	Panacea full (video+matrix+code): -p or --pana or --panacea
		Panacea video: --pv or --pvideo or --panav or --panavideo or --panaceav or --panaceavideo
		Panacea matrix: --pm or --pmatrix or --panam or --panamatrix or --panaceam or --panaceamatrix
		Panacea code: --pcode or --panac or --panacode or --panaceac or --panaceacode
		Panacea-youth: --py or --pyouth or --pmolodost or --panay or --panayouth or --panamolodost or --panaceay or --panaceayouth or --panaceamolodost
	Life Bracelets:
		Life Bracelet 1.0 Lite: --bj1 or --lb1 or --bj1l or --bj1lite or --lb1l or --lb1lite
		Life Bracelet 2.1: --bj21 or --lb21
	Life Bracelets 3.0:
		Life Bracelet 3.0 Lite: --bj30l or --bj30lite or --lb30l or --lb30lite or --bj3l or --bj3lite or --lb3l or --lb3lite
		Life Bracelet 3.0 Normal: --bj30n or --bj30normal or --lb30n or --lb30normal or --bj3n or --bj3normal or --lb3n or --lb3normal
		Special Zone full (matrix+video+Special Zone 3): --oz or --sz or --specialzone
			Special Zone matrix: --ozm or --ozmatrix or --szm or --szmatrix or --specialzonem or --specialzonematrix
			Special Zone video: --ozv or --ozvideo or --szv or --szvideo or --specialzonev or --specialzonevideo
			Special Zone 3: --oz3 or --sz3 or --specialzone3
	Society:
		Spectrum full (matrix+video): --spectr or --spektr or --spectrum
			Spectrum matrix: --spectrm or --spektrm or --spectrumm
			Spectrum video: --spectrv or --spektrv or --spectrumv
		AT: --at
		AT5: --at5
	Translations:
		Soul: --dusha or --soul
		Inner child: --innerchild or --child
		Elven: --elf or --elven
	Specific: --spec or --specific or --special
	Life Bracelets modules:
		Etalon: --etalon
		Tablet: --tablet or --tabletka or --pill
		Blood cleaning: --blood or --bloodcleaning or --krov or --chistkakrovi
		Probiotic: --probiotic or --probiotik
		Ozone: --ozon or --ozone
	Energy:
		Wheel: --wheel or --koleso
		Whirligig: --whirligig or --volchok
	Stabilizer programs:
		Stop-magic: --sm or --stopmagic or --stop or --stopm or --stopmagiya
		Health: --health or --zdorovie
		Protecion: --protection or --zaschita or --zashita or --zachita
		Stabilization: --stab or --stabilization or --stabil or --stabilizatsiya
	PC:
		PC: --pc or --pk
		AMB: --amb
Output directory (by default `output` inside the generator directory): --output=dirname
Language (value of the local, by default system local): --lang=[locale]
Silent mode (without processing log): -s or --silent
Debug mode (only for developers): --debug
See this help: -h or --help
Version: -v or --version
Developer contact: saband@quantummagic.org
		'''
}

__QM_DIR = os.path.dirname(os.path.abspath(__file__)) + '/'

__DEFAULT_OUTPUT = __QM_DIR+'output/'
__DEFAULT_THREADS = 10
__DEFAULT_OPACITY = 10

def getIntegerOption(opt, limits=(0,1024), defatulValue=0, message=''):
	if not opt:
		return defatulValue
	try:
		opt = int(opt)
	except ValueError:
		if message and message in __MESSAGES.keys():
			print __MESSAGES[message][lang]+opt
		print __MESSAGES['default'][lang]+str(defatulValue)
		return defatulValue
	if opt < limits[0] or opt > limits[1]:
		if message and message in __MESSAGES.keys():
			print __MESSAGES[message][lang]+opt
		print __MESSAGES['default'][lang]+str(defatulValue)
		return defatulValue
	return opt

def callAction(filename, option, opacity):
	PIPE = subprocess.PIPE
	if debug:
		print __QM_DIR+'engine.py --file="'+filename.replace('"','\\"')+'" -o '+str(opacity)+' --output="'+output.replace('"','\\"')+'" --lang='+lang+' '+option
	p = subprocess.Popen(__QM_DIR+'engine.py --file="'+filename.replace('"','\\"')+'" -o '+str(opacity)+' --output="'+output.replace('"','\\"')+'" --lang='+lang+' '+option, shell=True, stdin=PIPE, stdout=PIPE, stderr=subprocess.STDOUT, close_fds=(False if os.name == 'nt' else True))
	return p
	
def getAnswer(thread):
	out = thread[0].stdout.read().strip()
	if not out:
		print __MESSAGES['unexpected'][lang]+thread[1]
	else:
		regexpOut = re.compile('^done;(.+)$')
		matchOut = regexpOut.search(out)
		if matchOut:
			if not silent:
				print __MESSAGES['done'][lang]+matchOut.groups()[0]
		elif out in ('usage', 'sourcenotfound', 'nomodule'):
			print __MESSAGES[out][lang]
		elif out in __MESSAGES.keys():
			print __MESSAGES[out][lang]+thread[1]
		else:
			if debug:
				print out
			print __MESSAGES['unexpected'][lang]+thread[1]

def loadFileByURL(url, filename=''):
	if not silent:
		print __MESSAGES['urlloading'][lang]+url+(' => '+filename if filename else '')
	r = None
	try:
		r = requests.get(url, stream=True)
	except requests.exceptions:
		print __MESSAGES['urlnotfound'][lang]+url
		return False
	if not r.status_code == 200:
		print __MESSAGES['urlnotfound'][lang]+url
		return False
	if not r.headers.get('content-type') or not r.headers['content-type'] in ('image/png', 'image/gif', 'image/jpg', 'image/jpeg'):
		print __MESSAGES['unsupportedformat'][lang]+url
		return False
	filename = output + (filename if filename else os.path.basename(url))
	try:
		f = open(filename, 'wb')
		shutil.copyfileobj(r.raw, f)
		f.close()
	except Exception:
		print __MESSAGES['cannotwritetofile'][lang]+filename
	return filename

def loadMapsByCoords(coords):
	mapsSources = opts.get('--sources')
	if not mapsSources:
		mapsSources = 'yandex,google'
	if not mapsSources in ('yandex', 'google', 'yandex,google', 'google,yandex'):
		print __MESSAGES['incorrectsources'][lang]+mapsSources
		mapsSources = 'yandex,google'
		print __MESSAGES['default'][lang]+mapsSources
	zoom = opts.get('--zoom')
	if not zoom:
		zoom = '21,0'
	regexpZoom = re.compile('^([0-9]{1,2}),([0-9]{1,2})$')
	matchZoom = regexpZoom.search(zoom)
	if not matchZoom:
		print __MESSAGES['incorrectzoom'][lang]+zoom
		print __MESSAGES['default'][lang]+zoom
		zoom = (21,0)
	else:
		zoom = (int(matchZoom.groups(0)[0]), int(matchZoom.groups(0)[1]))
	if zoom[0] < zoom[1]:
		zoom = (zoom[1], zoom[0])
	if zoom[0] > 21 or zoom[0] < 0 or zoom[1] > 21 or zoom[1] < 0:
		print __MESSAGES['incorrectzoom'][lang]+str(zoom)
		print __MESSAGES['default'][lang]+str(zoom)
		zoom = (21,0)
	if zoom[0] > 17 and not 'google' in mapsSources:
		print __MESSAGES['nomaps'][lang]+str(zoom)
		return False
	maps = []
	if 'google' in mapsSources:
		for i in range(zoom[1], zoom[0]+1):
			loadedMap = loadFileByURL('http://maps.googleapis.com/maps/api/staticmap?center='+coords+'&size=640x640&maptype=satellite&sensor=false&zoom='+str(i), coords+'_google_'+str(i)+'.png')
			if loadedMap:
				maps.append(loadedMap)
	if 'yandex' in mapsSources:
		coords = ','.join((coords.split(',')[1],coords.split(',')[0]))
		for i in range(zoom[1], zoom[0]+1 if zoom[0] < 18 else 18):
			loadedMap = loadFileByURL('https://static-maps.yandex.ru/1.x/?z='+str(i)+'&size=650,450&l=sat&ll='+coords, coords+'_yandex_'+str(i)+'.jpeg')
			if loadedMap:
				maps.append(loadedMap)
	return maps
			
lang = locale.getdefaultlocale()[0]
if lang is None or not lang or not lang in __LANGS:
	lang = __DEFAULT_LANG

try:
	gOpts, gArgs = getopt.getopt(sys.argv[1:], "hvsf:d:l:c:u:o:t:p", ["help","version","silent","debug","file=","dir=","list=","url=","coords=","lang=","opacity=","threads=","output=","sources=","zoom="]+__GETOPT_OPTIONS)
except getopt.GetoptError:
	print __MESSAGES['usage'][lang]
	exit()

opts = {}
useOpts = []

for opt, arg in gOpts:
	opts[opt] = arg
	if opt in __OPTIONS:
		useOpts.append(opt)

if not opts.get('--lang') is None and opts.get('--lang') and opts.get('--lang') in __LANGS:
	lang = opts.get('--lang')

if not opts.get('-v') is None or not opts.get('--version') is None:
	print __VERSIONS[lang]+__VERSION+'.'
	exit()

if not opts.get('-h') is None or not opts.get('--help') is None:
	print __HELP[lang]
	exit()

silent = True if not opts.get('-s') is None or not opts.get('--silent') is None else False
debug = True if not opts.get('--debug') is None else False
opacity = getIntegerOption(opts.get('-o') if opts.get('-o') else opts.get('--opacity'), (0,100), __DEFAULT_OPACITY, 'opacity')
threads = getIntegerOption(opts.get('-t') if opts.get('-t') else opts.get('--threads'), (1,1024), __DEFAULT_THREADS, 'threads')
output = opts.get('--output')
if not output:
	output = __DEFAULT_OUTPUT
if not output[-1] == '/':
	output += '/'
if not os.path.isdir(output):
	try:
		os.makedirs(output)
	except Exception:
		print __MESSAGES['dirnotfoundandcannotbecreated'][lang]+output
		exit()
	if not os.path.isdir(output):
		print __MESSAGES['dirnotfoundandcannotbecreated'][lang]+output
		exit()

gFile = opts.get('-f') if opts.get('-f') else opts.get('--file')
gDir = opts.get('-d') if opts.get('-d') else opts.get('--dir')
gURL = opts.get('-u') if opts.get('-u') else opts.get('--url')
gList = opts.get('-l') if opts.get('-l') else opts.get('--list')
gCoords = opts.get('-c') if opts.get('-c') else opts.get('--coords')

if not gFile and not gDir and not gURL and not gList and not gCoords:
	print __MESSAGES['usage'][lang]
	exit()

print __MESSAGES['start'][lang]

if gFile:
	filename = gFile
	if not os.path.isfile(filename):
		print __MESSAGES['filenotfound'][lang]+filename
	else:
		activeThreads = []
		for opt in useOpts:
			if len(activeThreads) >= threads:
				getAnswer(activeThreads[0])
				del activeThreads[0]
			activeThreads.append([callAction(filename, opt, opacity), filename])
		while len(activeThreads):
			getAnswer(activeThreads[0])
			del activeThreads[0]
if gDir:
	dirname = gDir
	if not dirname[-1] == '/':
		dirname += '/'
	if dirname == output:
		print __MESSAGES['inputcannotbeoutput'][lang]+dirname
	elif not os.path.isdir(dirname):
		print __MESSAGES['dirnotfound'][lang]+dirname
	else:
		lsdir = os.listdir(dirname)
		activeThreads = []
		f = False
		for i in lsdir:
			if os.path.isfile(dirname+i):
				f = True
				for opt in useOpts:
					if len(activeThreads) >= threads:
						getAnswer(activeThreads[0])
						del activeThreads[0]
					activeThreads.append([callAction(dirname+i, opt, opacity), dirname+i])
		while len(activeThreads):
			getAnswer(activeThreads[0])
			del activeThreads[0]
		if not f:
			print __MESSAGES['dirisempty'][lang]+dirname
if gURL:
	filename = loadFileByURL(gURL)
	if filename:
		activeThreads = []
		toRemove = []
		for opt in useOpts:
			if len(activeThreads) >= threads:
				getAnswer(activeThreads[0])
				del activeThreads[0]
			activeThreads.append([callAction(filename, opt, opacity), filename])
			if not filename in toRemove:
				toRemove.append(filename)
		while len(activeThreads):
			getAnswer(activeThreads[0])
			del activeThreads[0]
		for i in toRemove:
			try:
				os.unlink(i)
			except Exception:
				pass
if gList:
	listfilename = gList
	if not os.path.isfile(listfilename):
		print __MESSAGES['filenotfound'][lang]+listfilename
	elif not mimetypes.guess_type(listfilename)[0] == 'text/plain':
		print __MESSAGES['listfilenottxt'][lang]+listfilename
	else:
		listFile = None
		try:
			listFile = open(listfilename)
		except Exception:
			print __MESSAGES['cannotopenfile'][lang]+listfilename
		if listFile:
			activeThreads = []
			toRemove = []
			regexpURL = re.compile('^http(s?)://(.+)$', re.I)
			regexpCoords = re.compile('^([+-]?)([0-9.]+),([+-]?)([0-9.]+)$')
			for line in listFile:
				line = line.strip()
				if os.path.isfile(line):
					for opt in useOpts:
						if len(activeThreads) >= threads:
							getAnswer(activeThreads[0])
							del activeThreads[0]
						activeThreads.append([callAction(line, opt, opacity), line])
				else:
					matchURL = regexpURL.search(line)
					matchCoords = regexpCoords.search(line)
					if matchURL:
						filename = loadFileByURL(line)
						if filename:
							for opt in useOpts:
								if len(activeThreads) >= threads:
									getAnswer(activeThreads[0])
									del activeThreads[0]
								activeThreads.append([callAction(filename, opt, opacity), filename])
								if not filename in toRemove:
									toRemove.append(filename)
					elif matchCoords:
						maps = loadMapsByCoords(line)
						if maps:
							for filename in maps:
								for opt in useOpts:
									if len(activeThreads) >= threads:
										getAnswer(activeThreads[0])
										del activeThreads[0]
									activeThreads.append([callAction(filename, opt, opacity), filename])
									if not filename in toRemove:
										toRemove.append(filename)
					else:
						print __MESSAGES['notfilenoturlnotcoords'][lang]+line
			while len(activeThreads):
				getAnswer(activeThreads[0])
				del activeThreads[0]
			for i in toRemove:
				try:
					os.unlink(i)
				except Exception:
					pass
if gCoords:
	maps = loadMapsByCoords(gCoords)
	if maps:
		activeThreads = []
		toRemove = []
		for filename in maps:
			for opt in useOpts:
				if len(activeThreads) >= threads:
					getAnswer(activeThreads[0])
					del activeThreads[0]
				activeThreads.append([callAction(filename, opt, opacity), filename])
				if not filename in toRemove:
					toRemove.append(filename)
		while len(activeThreads):
			getAnswer(activeThreads[0])
			del activeThreads[0]
		for i in toRemove:
			try:
				os.unlink(i)
			except Exception:
				pass
			
print __MESSAGES['exit'][lang]
exit()