from __future__ import division
try:
	import os, math
	from PIL import Image, ImageDraw, ImageFont
except ImportError as inst:
	print 'nomodule'
	exit()

__QM_DIR = os.path.dirname(os.path.abspath(__file__)) + '/'

def openImageRGBA(filename):
	if not os.path.isfile(filename):
		raise QMIGException('filenotfound')
	try:
		image = Image.open(filename)
	except IOError:
		raise QMIGException('unsupportedformat')
	return image.convert('RGBA')

def createImageResizedToMinSize(imageFileName, newImageSize):
	try:
		image = openImageRGBA(imageFileName)
	except QMIGException:
		raise QMIGException('sourcenotfound')
	x_pr = (newImageSize[0] / image.size[0]) if (image.size[0] > newImageSize[0]) else 1
	y_pr = (newImageSize[1] / image.size[1]) if (image.size[1] > newImageSize[1]) else 1
	pr = x_pr if (x_pr <= y_pr) else y_pr
	calculatedSize = (int(math.ceil(image.size[0]*pr)),int(math.ceil(image.size[1]*pr)))
	return image.resize(calculatedSize)

def createOverlayImageUseFilling(overlaySourceFilename, processedImageSize, useCeil=True):
	overlayImage = createImageResizedToMinSize(overlaySourceFilename, processedImageSize)
	wStep = overlayImage.size[0]
	hStep = overlayImage.size[1]
	wCount = int(math.ceil(processedImageSize[0]/wStep) if useCeil else math.floor(processedImageSize[0]/wStep))
	hCount = int(math.ceil(processedImageSize[1]/hStep) if useCeil else math.floor(processedImageSize[1]/hStep))
	filledImage = Image.new('RGBA', (processedImageSize[0],processedImageSize[1]) if useCeil else (wStep*wCount,hStep*hCount))
	for h in range(0, hCount):
		for w in range(0, wCount):
			filledImage.paste(overlayImage, (w*wStep,h*hStep))
	return filledImage

def blendImages(firstImage, secondImage, opacity):
	return Image.blend(firstImage, secondImage, opacity)

def printTextOnImage(image, text, coords, opacity, fontsize):
	draw = ImageDraw.Draw(image)
	try:
		font = ImageFont.truetype(__QM_DIR+'sources/DejaVuSans.ttf', fontsize)
	except Exception:
		raise QMIGException('sourcenotfound')
	draw.text(coords, unicode(text), (0,0,0,int(math.ceil(opacity*255))), font=font)

class QMIGException(Exception):
	pass